
/* 
 * This macro was used to generate histograms 
 * appearing in 2nd 20F proposal for IGISOL (submitted in Sep 2016).
 *
 * source options:
 *  - muon
 *  - forbidden
 *  - allowed
 *  - summing
 *  - pileup
 *  - bi207
 */

void histo( TString filename, TString source, Double_t emin = 5600, Double_t emax = 7000, Double_t hours = 168, bool wait = false)   // 168 hours = 7 days
{
    // set some parameters
    Double_t muonFlux  = 0.93;       // cm^-2 min^-1  (old value: 0.68)
    Double_t days      = hours/24;	 // days
    Double_t rate      = 40e3;       // Hz
    Double_t vetoThres = 100;        // keV
    Double_t branching = 1e-6;
    Double_t dtPileup = 20;          // ns
    Double_t vetoEfficiency = 0.99;
	Double_t bi207act = 3.4e5;	 	 // Activity in Bq
	Double_t bi207int = 0.0708;	     // Intensity of 976-keV conversion-electron line
    
    // Open root file
    TFile *f = TFile::Open(filename);
    
    // Get TTree
    TTree *t = (TTree*)f -> Get("Detector");

    // Normalization
    Double_t norm = ((TParameter<double>*)(t->GetUserInfo()->FindObject("NORMALIZATION")))->GetVal();

	// Calculate scaling factor
	Double_t sf = 1;
	if (source == "muon")
    {
        Double_t area = ((TParameter<double>*)
                            (t->GetUserInfo()->FindObject("SOURCE_AREA/cm2")))->GetVal();
        Double_t minutes = days*24*60;
        Double_t muons = muonFlux * minutes * area;
        sf = muons / norm;
    }
	else if (source == "bi207") 
	{
        Double_t seconds = days*24*60*60;
        Double_t electrons = bi207act * bi207int * seconds;
        sf = electrons / norm;
	}
    else
    {
        Double_t seconds = days*24*60*60;
        Double_t decays = rate * seconds;
        sf = decays / norm;
        if (source=="pileup")     sf *= dtPileup * rate * 1e-9;
        else if (source=="forbidden")  sf *= branching;
    }    
    
    cout << "\n Scaling factor: " << sf << "\n" << endl;
    
    
    // Now make some plots

    // Canvas
    TCanvas *canvas = new TCanvas("c1", "multipads", 900, 700);
    canvas -> Clear();

    // energy in signal
    TH1F *hSingles = new TH1F("hSingles", "hSingles", 300, 0., 15000.);
    t -> Draw("EDepSignal >> hSingles"); 
    hSingles -> Scale(sf);
    canvas->Update();
    if (wait) gPad->WaitPrimitive();

    // use veto shield
    TH1F *hVetoed = new TH1F("hVetoed", "hVetoed", 300, 0., 15000.);
    TCut veto = Form("EDepVeto < %.1f", vetoThres);
    t -> Draw("EDepSignal >> hVetoed", veto); 
    hVetoed -> Scale(sf);
    TH1F *hBackgr = (TH1F*)hSingles->Clone();
    hBackgr -> Scale(1.-vetoEfficiency);
    hVetoed -> Add(hBackgr);
    canvas->Update();
    
    // "detach" the histograms from file and close file
    hSingles -> SetDirectory(0); 
    hVetoed -> SetDirectory(0); 
    f->Close();
    
    cout << " Counts between " << Form("%.2f",emin/1000.) << " MeV and ";
    cout << Form("%.2f",emax/1000.) << " MeV:" << endl;
    binx1 = hVetoed -> GetXaxis() -> FindBin(emin);
    binx2 = hVetoed -> GetXaxis() -> FindBin(emax);
    cout << "  - w/o veto:  " << hSingles -> Integral(binx1, binx2) << endl;
    cout << "  - with veto: " << hVetoed -> Integral(binx1, binx2) << endl << endl;
    
    // write histograms to new file
    TString fname = source + "_his.root";
    TFile *fo = TFile::Open(fname, "recreate");
    hSingles -> Write();
    hVetoed -> Write();
    fo -> Close();

	cout << " Histograms written to: " << fname << endl << endl;
}
